@@ -1,4 +1,5 @@
 import asyncio
+import random
 
 import dask
 from dask_cloudprovider.generic.vmcluster import (
@@ -13,9 +14,11 @@ from dask_cloudprovider.aws.helper import (
     get_vpc_subnets,
     get_security_group,
 )
+from dask_cloudprovider.utils.timeout import Timeout
 
 try:
     import aiobotocore
+    import botocore.exceptions
 except ImportError as e:
     msg = (
         "Dask Cloud Provider AWS requirements are not installed.\n\n"
@@ -33,6 +36,7 @@ class EC2Instance(VMInterface):
         config,
         *args,
         region=None,
+        availability_zone=None,
         bootstrap=None,
         ami=None,
         docker_image=None,
@@ -43,6 +47,8 @@ class EC2Instance(VMInterface):
         subnet_id=None,
         security_groups=None,
         filesystem_size=None,
+        key_name=None,
+        iam_instance_profile=None,
         **kwargs,
     ):
         super().__init__(*args, **kwargs)
@@ -50,6 +56,7 @@ class EC2Instance(VMInterface):
         self.cluster = cluster
         self.config = config
         self.region = region
+        self.availability_zone = availability_zone
         self.bootstrap = bootstrap
         self.ami = ami
         self.docker_image = docker_image or self.config.get("docker_image")
@@ -60,6 +67,8 @@ class EC2Instance(VMInterface):
         self.subnet_id = subnet_id
         self.security_groups = security_groups
         self.filesystem_size = filesystem_size
+        self.key_name = key_name
+        self.iam_instance_profile = iam_instance_profile
 
     async def create_vm(self):
         """
@@ -84,8 +93,8 @@ class EC2Instance(VMInterface):
                 "099720109477",  # Canonical
             )
 
-            response = await client.run_instances(
-                BlockDeviceMappings=[
+            vm_kwargs = {
+                "BlockDeviceMappings": [
                     {
                         "DeviceName": "/dev/sda1",
                         "VirtualName": "sda1",
@@ -97,20 +106,14 @@ class EC2Instance(VMInterface):
                         },
                     }
                 ],
-                ImageId=self.ami,
-                InstanceType=self.instance_type,
-                MaxCount=1,
-                MinCount=1,
-                Monitoring={"Enabled": False},
-                UserData=self.cluster.render_cloud_init(
-                    image=self.docker_image,
-                    command=self.command,
-                    gpu_instance=self.gpu_instance,
-                    bootstrap=self.bootstrap,
-                    env_vars=self.env_vars,
-                ),
-                InstanceInitiatedShutdownBehavior="terminate",
-                NetworkInterfaces=[
+                "ImageId": self.ami,
+                "InstanceType": self.instance_type,
+                "MaxCount": 1,
+                "MinCount": 1,
+                "Monitoring": {"Enabled": False},
+                "UserData": self.cluster.render_process_cloud_init(self),
+                "InstanceInitiatedShutdownBehavior": "terminate",
+                "NetworkInterfaces": [
                     {
                         "AssociatePublicIpAddress": True,
                         "DeleteOnTermination": True,
@@ -120,7 +123,20 @@ class EC2Instance(VMInterface):
                         "SubnetId": self.subnet_id,
                     }
                 ],
-            )
+            }
+
+            if self.key_name:
+                vm_kwargs["KeyName"] = self.key_name
+
+            if self.iam_instance_profile:
+                vm_kwargs["IamInstanceProfile"] = self.iam_instance_profile
+
+            if self.availability_zone:
+                if isinstance(self.availability_zone, list):
+                    self.availability_zone = random.choice(self.availability_zone)
+                vm_kwargs["Placement"] = {"AvailabilityZone": self.availability_zone}
+
+            response = await client.run_instances(**vm_kwargs)
             [self.instance] = response["Instances"]
             await client.create_tags(
                 Resources=[self.instance["InstanceId"]],
@@ -133,16 +149,27 @@ class EC2Instance(VMInterface):
                 f"Created instance {self.instance['InstanceId']} as {self.name}"
             )
 
+            timeout = Timeout(
+                300,
+                f"Failed Public IP for instance {self.instance['InstanceId']}",
+            )
             while (
                 "PublicIpAddress" not in self.instance
                 or self.instance["PublicIpAddress"] is None
-            ):
-                await asyncio.sleep(0.1)  # TODO back off correctly
-                response = await client.describe_instances(
-                    InstanceIds=[self.instance["InstanceId"]], DryRun=False
-                )
-                [reservation] = response["Reservations"]
-                [self.instance] = reservation["Instances"]
+            ) and timeout.run():
+                backoff = 0.1
+                await asyncio.sleep(
+                    min(backoff, 10) + backoff % 1
+                )  # Exponential backoff with a cap of 10 seconds and some jitter
+                try:
+                    response = await client.describe_instances(
+                        InstanceIds=[self.instance["InstanceId"]], DryRun=False
+                    )
+                    [reservation] = response["Reservations"]
+                    [self.instance] = reservation["Instances"]
+                except botocore.exceptions.ClientError as e:
+                    timeout.set_exception(e)
+                backoff = backoff * 2
             return self.instance["PublicIpAddress"]
 
     async def destroy_vm(self):
@@ -182,6 +209,10 @@ class EC2Cluster(VMCluster):
     ----------
     region: string (optional)
         The region to start you clusters. By default this will be detected from your config.
+    availability_zone: string or List(string) (optional)
+        The availability zone to start you clusters. By default AWS will select the AZ with most free capacity.
+        If you specify more than one then scheduler and worker VMs will be randomly assigned to one of your
+        chosen AZs.
     bootstrap: bool (optional)
         It is assumed that the ``ami`` will not have Docker installed (or the NVIDIA drivers for GPU instances).
         If ``bootstrap`` is ``True`` these dependencies will be installed on instance start. If you are using
@@ -223,6 +254,16 @@ class EC2Cluster(VMCluster):
         The instance filesystem size in GB.
 
         Defaults to ``40``.
+    key_name: str (optional)
+        The SSH key name to assign to all instances created by the cluster manager.
+        You can list your existing key pair names with
+        ``aws ec2 describe-key-pairs  --query 'KeyPairs[*].KeyName' --output text``.
+
+        NOTE: You will need to ensure your security group allows access on port 22. If ``security_groups``
+        is not set the default group will not contain this rule and you will need to add it manually.
+    iam_instance_profile: dict (optional)
+        An IAM profile to assign to VMs. This can be used for allowing access to other AWS resources such as S3.
+        See https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html.
     n_workers: int
         Number of workers to initialise the cluster with. Defaults to ``0``.
     worker_module: str
@@ -244,6 +285,8 @@ class EC2Cluster(VMCluster):
         For GPU instance types the Docker image much have NVIDIA drivers and ``dask-cuda`` installed.
 
         By default the ``daskdev/dask:latest`` image will be used.
+    docker_args: string (optional)
+        Extra command line arguments to pass to Docker.
     env_vars: dict (optional)
         Environment variables to be passed to the worker.
     silence_logs: bool
@@ -254,7 +297,7 @@ class EC2Cluster(VMCluster):
     security : Security or bool, optional
         Configures communication security in this cluster. Can be a security
         object, or True. If True, temporary self-signed credentials will
-        be created automatically.
+        be created automatically. Default is ``True``.
 
     Notes
     -----
@@ -269,6 +312,28 @@ class EC2Cluster(VMCluster):
         EC2 Instance, dask-worker-{cluster uuid}-{worker uuid}, Dask Workers, "`EC2 Pricing
         <https://aws.amazon.com/ec2/pricing/>`_"
 
+    **Credentials**
+
+    In order for Dask workers to access AWS resources such as S3 they will need credentials.
+
+    The best practice way of doing this is to pass an IAM role to be used by workers. See the ``iam_instance_profile``
+    keyword for more information.
+
+    Alternatively you could read in your local credentials created with ``aws configure`` and pass them along
+    as environment variables. Here is a small example to help you do that.
+
+    >>> def get_aws_credentials():
+    ...     parser = configparser.RawConfigParser()
+    ...     parser.read(os.path.expanduser('~/.aws/config'))
+    ...     config = parser.items('default')
+    ...     parser.read(os.path.expanduser('~/.aws/credentials'))
+    ...     credentials = parser.items('default')
+    ...     all_credentials = {key.upper(): value for key, value in [*config, *credentials]}
+    ...     with contextlib.suppress(KeyError):
+    ...     all_credentials["AWS_REGION"] = all_credentials.pop("REGION")
+    ...     return all_credentials
+    >>> cluster = EC2Cluster(env_vars=get_aws_credentials())
+
     **Manual cleanup**
 
     If for some reason the cluster manager is terminated without being able to perform cleanup
@@ -297,17 +362,35 @@ class EC2Cluster(VMCluster):
 
     RAPIDS Cluster.
 
-    >>> cluster = EC2Cluster(ami="ami-0c7c7d78f752f8f17",  # Example Deep Learning AMI (Ubuntu 18.04)
-                             docker_image="rapidsai/rapidsai:cuda10.1-runtime-ubuntu18.04",
-                             instance_type="p3.2xlarge",
-                             worker_module="dask_cuda.cli.dask_cuda_worker",
-                             bootstrap=False,
-                             filesystem_size=120)
+
+    >>> env_vars =
+    >>> cluster = EC2Cluster(ami="ami-06d62f645899df7de",  # Deep Learning AMI Ubuntu 18.04 (these are region specific)
+    ...                      docker_image="rapidsai/rapidsai:cuda11.0-runtime-ubuntu18.04",
+    ...                      instance_type="g4dn.xlarge",
+    ...                      worker_class="dask_cuda.CUDAWorker",
+    ...                      n_workers=2,
+    ...                      bootstrap=False,
+    ...                      filesystem_size=120,
+    ...                      env_vars=get_aws_credentials()) # Pass credentials to Cluster see Notes section for info
+=======
+
+    Enable SSH for debugging
+
+    >>> from dask_cloudprovider.aws import EC2Cluster
+    >>> cluster = EC2Cluster(key_name="myawesomekey",
+                             # Security group which allows ports 22, 8786, 8787 and all internal traffic
+                             security_groups=["sg-aabbcc112233"])
+
+    # You can now SSH to an instance with `ssh ubuntu@public_ip`
+
+    >>> cluster.close()
+
     """
 
     def __init__(
         self,
         region=None,
+        availability_zone=None,
         bootstrap=None,
         auto_shutdown=None,
         ami=None,
@@ -316,6 +399,8 @@ class EC2Cluster(VMCluster):
         subnet_id=None,
         security_groups=None,
         filesystem_size=None,
+        key_name=None,
+        iam_instance_profile=None,
         docker_image=None,
         **kwargs,
     ):
@@ -324,6 +409,11 @@ class EC2Cluster(VMCluster):
         self.scheduler_class = EC2Scheduler
         self.worker_class = EC2Worker
         self.region = region if region is not None else self.config.get("region")
+        self.availability_zone = (
+            availability_zone
+            if availability_zone is not None
+            else self.config.get("availability_zone")
+        )
         self.bootstrap = (
             bootstrap if bootstrap is not None else self.config.get("bootstrap")
         )
@@ -341,7 +431,7 @@ class EC2Cluster(VMCluster):
         self.gpu_instance = self.instance_type.startswith(("p", "g"))
         self.vpc = vpc if vpc is not None else self.config.get("vpc")
         self.subnet_id = (
-            subnet_id if subnet_id is None else self.config.get("subnet_id")
+            subnet_id if subnet_id is not None else self.config.get("subnet_id")
         )
         self.security_groups = (
             security_groups
@@ -354,10 +444,19 @@ class EC2Cluster(VMCluster):
             else self.config.get("filesystem_size")
         )
 
+        self.key_name = (
+            key_name if key_name is not None else self.config.get("key_name")
+        )
+        self.iam_instance_profile = (
+            iam_instance_profile
+            if iam_instance_profile is not None
+            else self.config.get("iam_instance_profile")
+        )
         self.options = {
             "cluster": self,
             "config": self.config,
             "region": self.region,
+            "availability_zone": self.availability_zone,
             "bootstrap": self.bootstrap,
             "ami": self.ami,
             "docker_image": docker_image or self.config.get("docker_image"),
@@ -367,6 +466,8 @@ class EC2Cluster(VMCluster):
             "subnet_id": self.subnet_id,
             "security_groups": self.security_groups,
             "filesystem_size": self.filesystem_size,
+            "key_name": self.key_name,
+            "iam_instance_profile": self.iam_instance_profile,
         }
         self.scheduler_options = {**self.options}
         self.worker_options = {**self.options}
